

'use strict';
var utils = require('./utils');
var removeUndefined = require('./parse/utils').removeUndefined;
var validate = utils.common.validate;
var composeAsync = utils.common.composeAsync;

function formatAccountInfo(response) {
  var data = response.account_data;
  return removeUndefined({
    sequence: data.Sequence,
    xrpBalance: utils.common.dropsToXrp(data.Balance),
    ownerCount: data.OwnerCount,
    previousInitiatedTransactionID: data.AccountTxnID,
    previousAffectingTransactionID: data.PreviousTxnID,
    previousAffectingTransactionLedgerVersion: data.PreviousTxnLgrSeq
  });
}

function getAccountInfo(account, options, callback) {
  validate.address(account);
  validate.getAccountInfoOptions(options);

  var request = {
    account: account,
    ledger: options.ledgerVersion
  };

  this.remote.requestAccountInfo(request, composeAsync(formatAccountInfo, callback));
}

module.exports = utils.wrapCatch(getAccountInfo);